using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;

namespace Azbest
{
    public partial class EternityForm : Form
    {
        private static EternityForm instance = null;
        public const string SettingsFile = "settings.txt";

        public static EternityForm Instance
        {
            get { return instance; }
        }

        public EternityForm()
        {
            InitializeComponent();
            if (!groupBoxBoard.Controls.Contains(boardControl))
            {
                groupBoxBoard.Controls.Add(boardControl);
            }
            if (!groupBoxTiles.Controls.Contains(groupBoxTiles))
            {
                groupBoxTiles.Controls.Add(tilesControl);
            }
            ResizeFormIfNeeded();
            instance = this;
            if (File.Exists(SettingsFile))
            {
                string [] settings = File.ReadAllLines(SettingsFile);
                textBoxProgramPath.Text = settings[0];
            }
        }

        private void ResizeFormIfNeeded()
        {
            if (Screen.AllScreens[0].Bounds.Width == 1024)
            {
                groupBoxBoard.Width -= 140;
                groupBoxBoard.Height -= 140;
                groupBoxTiles.Width -= 140;
                groupBoxTiles.Height -= 140;

                this.tilesControl.Size -= new Size(140, 140);
                this.boardControl.Size -= new Size(140, 140);

                groupBoxTiles.Left -= 140;
                groupBoxActions.Left -= 140;
                this.Width -= 40;
                this.Height -= 60;
            }
        }

        private void GenerateBoard(bool useRandom, bool uniqueOnly)
        {
            int newBoardSize = trackBarBoardSize.Value;
            int newBoardColor = trackBarColors.Value;
            int newBoardBorderColor = trackBarColorsBorder.Value;
            GameManager.Instance.FreeBoardTiles.Tiles.Clear();
            GameManager.Instance.CurrentBoard.Generate(newBoardSize, newBoardColor, newBoardBorderColor, checkBoxShuffle.Checked, useRandom, uniqueOnly);
            if (!checkBoxShowResolved.Checked)
            {
                GameManager.Instance.CurrentBoard.MoveAllTilesToPool(GameManager.Instance.FreeBoardTiles);
            }

            Refresh();
        }

        private void buttonGenerateBoard_Click(object sender, EventArgs e)
        {
            GenerateBoard(this.checkBoxUseRandom.Checked, this.checkBoxUniqueOnly.Checked);
        }

        private void trackBarBoardSize_Scroll(object sender, EventArgs e)
        {
            labelBoardSize.Text = trackBarBoardSize.Value.ToString();
        }

        private void trackBarColors_Scroll(object sender, EventArgs e)
        {
            labelColors.Text = trackBarColors.Value.ToString();
        }

        private void buttonExportTiles_Click(object sender, EventArgs e)
        {
            FileDialog fd = new SaveFileDialog();
            if (fd.ShowDialog() != DialogResult.OK)
            {
                return;
            }
            GameManager.Instance.FreeBoardTiles.SaveToFile(fd.FileName);            
        }

        private void buttonImportTiles_Click(object sender, EventArgs e)
        {
            FileDialog fd = new OpenFileDialog();
            if (fd.ShowDialog() != DialogResult.OK)
            {
                return;
            }
            GameManager.Instance.FreeBoardTiles.LoadFromFile(fd.FileName);
            Refresh();
        }

        private void checkBoxShowResolved_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxShowResolved.Checked)
            {
                checkBoxShuffle.Checked = false;
                checkBoxShuffle.Enabled = false;
            }
            else
            {
                checkBoxShuffle.Enabled = true;
            }
        }

        private void buttonExportBoard_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Not implemented.");
        }

        private void buttonImportBoard_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Not implemented.");
        }

        private void buttonVerifyBoard_Click(object sender, EventArgs e)
        {
            if (GameManager.Instance.CurrentBoard.IsCompleted())
            {
                MessageBox.Show("Szacun na oce, plansza jest ok!");
            }
            else
            {
                MessageBox.Show("To uoenie nie jest poprawne.");
            }
        }

        private void buttonBrowsePath_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFile = new OpenFileDialog();
            openFile.RestoreDirectory = true;
            if (openFile.ShowDialog() == DialogResult.OK)
            {
                this.textBoxProgramPath.Text = openFile.FileName;
            }
        }

        private void buttonRunProgram_Click(object sender, EventArgs e)
        {
            RunProgram();
        }

        private void buttonListen_Click(object sender, EventArgs e)
        {
            ListenForPlayer();
        }

        private void ListenForPlayer()
        {
            GameManager.Instance.ExternalResolverControl.ListenForPlayer();
        }

        private void RunProgram()
        {
            GameManager.Instance.ExternalResolverControl.RunProgram(textBoxProgramPath.Text);
        }

        private void EternityForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            List<string> settings = new List<string>();
            settings.Add(textBoxProgramPath.Text);
            File.WriteAllLines(SettingsFile, settings.ToArray());
        }

        private void buttonStartAll_Click(object sender, EventArgs e)
        {
            GenerateBoard(this.checkBoxUseRandom.Checked, this.checkBoxUniqueOnly.Checked);
            ListenForPlayer();
            RunProgram();
        }

        private void trackBarColorsBorder_Scroll(object sender, EventArgs e)
        {
            labelBorderColors.Text = trackBarColorsBorder.Value.ToString();
        }

        private int enqueuedRefreshes = 0;

        private void DoEnqueuedRefresh()
        {
            Refresh();
            --enqueuedRefreshes;
        }

        public void EnqueueRefresh()
        {
            ++enqueuedRefreshes;
            if (enqueuedRefreshes > 1) return;
            if (InvokeRequired)
            {
                Invoke(new MethodInvoker(DoEnqueuedRefresh));
            }
            else
            {
                DoEnqueuedRefresh();
            }
        }
    }
}