using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Diagnostics;
using System.Threading;

namespace Azbest
{
    public class ExternalResolver
    {
        public TcpListener TcpServer = new TcpListener(IPAddress.Any, 2043);

        internal void RunProgram(string filename)
        {
            try
            {
                Process proc = new Process();
                proc.StartInfo.FileName = filename;
                proc.Start();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        internal void ListenForPlayer()
        {
            try
            {
                if (GameManager.Instance.FreeBoardTiles.Tiles.Count == 0)
                {
                    MessageBox.Show("Musisz najpierw wygenerowa pytki!");
                    return;
                }
                TcpServer.Start();
                TcpServer.BeginAcceptTcpClient(new AsyncCallback(ClientConnected), TcpServer);                
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        private void ClientThread(object arg)
        {
            try
            {
                TcpClient client = (TcpClient)arg;
                RotatedTile[] startTiles = new RotatedTile[GameManager.Instance.FreeBoardTiles.Tiles.Count];
                GameManager.Instance.FreeBoardTiles.Tiles.CopyTo(startTiles);

                if (client != null)
                {
                    NetworkStream stream = client.GetStream();
                    byte[] tileDesc = Encoding.UTF8.GetBytes("[" + Environment.NewLine + GameManager.Instance.FreeBoardTiles.GetTilesDescription() + "]");
                    stream.Write(tileDesc, 0, tileDesc.Length);
                    byte[] readBuffer = new byte[1024 * 1024];

                    string readData = "";
                    while (true)
                    {
                        
                        if (!(readData.Contains("[") && readData.Contains("]")))
                        {
                            string tempData = "";
                            while (!readData.Contains("]"))
                            {                                
                                int readCount = stream.Read(readBuffer, 0, readBuffer.Length);
                                tempData = Encoding.UTF8.GetString(readBuffer, 0, readCount);
                                readData += tempData;
                            }
                        }

                        string[] lines = readData.Split(Environment.NewLine.ToCharArray(), StringSplitOptions.RemoveEmptyEntries);
                        if ((lines == null) || (lines.Length == 0))
                        {
                            MessageBox.Show("Pusta wiadomo.");
                            break;
                        }
                        if (lines[0] != "[")
                        {
                            MessageBox.Show("Ze dane na wejciu - brak nawiasu otwierajcego.");
                            break;
                        }
                        int dataRows = int.Parse(lines[1]);
                        if (dataRows < 1)
                        {
                            //MessageBox.Show("Program wysa informacj o zakoczeniu.");
                            break;
                        }

                        int iX = 0;
                        int iY = 0;
                        lock (GameManager.Instance.CurrentBoard.BoardLock)
                        {
                            GameManager.Instance.FreeBoardTiles.Tiles.Clear();
                            GameManager.Instance.FreeBoardTiles.Tiles.AddRange(startTiles);
                            
                            for (int i = 0; i < dataRows; ++i)
                            {
                                string[] tilePos = lines[i + 2].Split(" ".ToCharArray(), StringSplitOptions.RemoveEmptyEntries);
                                int tileNum = int.Parse(tilePos[0]);
                                
                                if (tileNum > -1)
                                {
                                    RotatedTile rotatedTile = startTiles[tileNum];
                                    int tileRot = int.Parse(tilePos[1]);
                                    GameManager.Instance.CurrentBoard.Tiles[iX, iY] = rotatedTile;
                                    GameManager.Instance.CurrentBoard.Tiles[iX, iY].Rotation = tileRot;
                                    if (GameManager.Instance.FreeBoardTiles.Tiles.Contains(rotatedTile))
                                    {
                                        int oldIndex = GameManager.Instance.FreeBoardTiles.Tiles.IndexOf(rotatedTile);
                                        GameManager.Instance.FreeBoardTiles.Tiles.Remove(rotatedTile);
                                        GameManager.Instance.FreeBoardTiles.Tiles.Insert(oldIndex, null);
                                    }
                                }
                                else
                                {
                                    GameManager.Instance.CurrentBoard.Tiles[iX, iY] = null;
                                }
                                ++iX;
                                if (iX == GameManager.Instance.CurrentBoard.BoardSize)
                                {
                                    iX = 0;
                                    ++iY;
                                }
                            }
                        }
                        readData = readData.Substring(readData.IndexOf(']') + 1, (readData.Length - readData.IndexOf(']')) - 1).TrimStart("\n\r".ToCharArray());
                        EternityForm.Instance.EnqueueRefresh();

                        Application.DoEvents();
                    }
                    //MessageBox.Show("Komunikacja zostaa zakoczona.");
                }
            }
            catch
            {
                MessageBox.Show("Poczenie zostao zerwane.");
            }
        }

        private void ClientConnected(IAsyncResult ar)
        {
            TcpListener listener = (TcpListener)ar.AsyncState;
            TcpClient client = listener.EndAcceptTcpClient(ar);
            Thread clientThread = new Thread(new ParameterizedThreadStart(ClientThread));
            clientThread.Start(client);
        }
            
    }
}
