using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace Azbest
{
    public partial class TilesControl : UserControl
    {
        public TilesControl()
        {
            InitializeComponent();
        }

        private void TilesControl_Paint(object sender, PaintEventArgs e)
        {
            GameManager.Instance.FreeBoardTiles.Draw(e.Graphics, this.Width);
        }

        private void TilesControl_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                RotatedTile draggedTile = GameManager.Instance.FreeBoardTiles.GetTileAt(e.Location, this.Width);
                if (draggedTile != null)
                {
                    lastTakenTileIndex = GameManager.Instance.FreeBoardTiles.GetTileIndexAt(e.Location, this.Width);
                    this.DoDragDrop(draggedTile, DragDropEffects.Move);                
                }
            }
            if (e.Button == MouseButtons.Right)
            {
                RotatedTile tile = GameManager.Instance.FreeBoardTiles.GetTileAt(e.Location, this.Width);
                if (tile != null)
                {
                    tile.Rotate(3);
                    Refresh();
                }
            }
        }

        private void TilesControl_Load(object sender, EventArgs e)
        {
            GameManager.Instance.FreeBoardTiles.RefreshViewControl += new MethodInvoker(this.Refresh);
        }

        private int lastTakenTileIndex = 0;

        private void TilesControl_DragDrop(object sender, DragEventArgs e)
        {
            RotatedTile draggedTile = (RotatedTile)e.Data.GetData(typeof(RotatedTile));
            Point pointInControl = this.PointToClient(new Point(e.X, e.Y));
            if (GameManager.Instance.FreeBoardTiles.Tiles.Contains(draggedTile))
            {
                RotatedTile oldTile = GameManager.Instance.FreeBoardTiles.GetTileAt(pointInControl, this.Width);
                if (oldTile == null)
                {
                    int tileIndex = GameManager.Instance.FreeBoardTiles.GetTileIndexAt(pointInControl, this.Width);
                    GameManager.Instance.FreeBoardTiles.Tiles[tileIndex] = draggedTile;
                    GameManager.Instance.FreeBoardTiles.Tiles[lastTakenTileIndex] = null;
                    Refresh();
                    return;
                }
                else
                {
                    int tileIndex = GameManager.Instance.FreeBoardTiles.GetTileIndexAt(pointInControl, this.Width);

                    GameManager.Instance.FreeBoardTiles.Tiles[lastTakenTileIndex] = oldTile;
                    GameManager.Instance.FreeBoardTiles.Tiles[tileIndex] = draggedTile;
                    Refresh();
                    return;
                }
            }
            else
            {
                RotatedTile oldTile = GameManager.Instance.FreeBoardTiles.GetTileAt(pointInControl, this.Width);
                if (oldTile == null)
                {
                    int tileIndex = GameManager.Instance.FreeBoardTiles.GetTileIndexAt(pointInControl, this.Width);
                    GameManager.Instance.FreeBoardTiles.Tiles[tileIndex] = draggedTile;

                    Point oldIndex = GameManager.Instance.CurrentBoard.GetIndexOfTile(draggedTile);
                    GameManager.Instance.CurrentBoard.Tiles[oldIndex.X, oldIndex.Y] = null;

                    Parent.Refresh();
                    GameManager.Instance.CurrentBoard.RefreshView();
                    Refresh();
                    return;
                }
                Console.Beep();
                return;
            }

        }

        private void TilesControl_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.Move;
        }
    }
}
