using System;
using System.Collections.Generic;
using System.Text;

namespace Azbest
{
    public class GameTile : RotatedTile
    {
        public readonly static GameTile EmptyTile = new GameTile(new Tile(-1, -1, -1, -1), 0, -1, 0);
        public readonly static GameTile BorderTile = new GameTile(new Tile(0, 0, 0, 0), 0, -1, 0);
        
        public int Index;

        public int Uniqueness = 0;

        private int[] ColorUniquenessMultiplier = new int[4];

        public int ColorUniquenessMultiplierLeft
        {
            get { return ColorUniquenessMultiplier[(4 - Rotation) % 4]; }
        }

        public int ColorUniquenessMultiplierTop
        {
            get { return ColorUniquenessMultiplier[(5 - Rotation) % 4]; }
        }

        public int ColorUniquenessMultiplierRight
        {
            get { return ColorUniquenessMultiplier[(6 - Rotation) % 4]; }
        }

        public int ColorUniquenessMultiplierBottom
        {
            get { return ColorUniquenessMultiplier[(7 - Rotation) % 4]; }
        }

        public GameTile(Tile baseTile, int rotation, int index, int uniqueness) : base (baseTile, rotation)
        {
            Index = index;
            Uniqueness = uniqueness;
            for (int i = 0; i < 4; ++i)
            {
                bool alreadyUsed = false;
                for (int j = 0; j < i; ++j)
                {
                    if (BaseTile.SideColors[i] == BaseTile.SideColors[j])
                    {
                        alreadyUsed = true;
                        break;
                    }
                }
                if (alreadyUsed)
                {
                    ColorUniquenessMultiplier[i] = 0;
                }
                else
                {
                    ColorUniquenessMultiplier[i] = 1;
                }
            }
            Random rand = new Random();
            //Uniqueness = rand.Next() % 100000;
        }

        public bool MatchesPattern(int patternLeft, int patternUp, int patternRight, int patternDown)
        {
            if ((patternLeft != -1)&&(patternLeft!= LeftColor))
            {
                return false;
            }
            if ((patternUp != -1) && (patternUp != TopColor))
            {
                return false;
            }
            if ((patternRight != -1) && (patternRight != RightColor))
            {
                return false;
            }
            return (!((patternDown != -1) && (patternDown != BottomColor)));
        }

        public bool AnyRotationMatchesPattern(int patternLeft, int patternUp, int patternRight, int patternDown)
        {
            int prevRot = Rotation;
            for (int i = 0; i < 4; ++i)
            {
                Rotation = i;
                if (MatchesPattern(patternLeft, patternUp, patternRight, patternDown))
                {
                    Rotation = prevRot;
                    return true;
                }
            }
            Rotation = prevRot;
            return false;
        }

        public GameTile Clone()
        {
            return new GameTile(this.BaseTile, this.Rotation, this.Index, this.Uniqueness);
        }
    }

    public class GameTileUniquenessComparer : IComparer<GameTile>
    {
        #region IComparer<GameTile> Members

        public int Compare(GameTile x, GameTile y)
        {
            return x.Uniqueness - y.Uniqueness;
        }

        #endregion
    }

    public class GameTileUniquenessRevComparer : IComparer<GameTile>
    {
        #region IComparer<GameTile> Members

        public int Compare(GameTile x, GameTile y)
        {
            return y.Uniqueness - x.Uniqueness;
        }

        #endregion
    }
}
