#define USE_COLOR_COUNTING

using System;
using System.Collections.Generic;
using System.Text;

namespace Azbest
{
    // Array based
    public class TileFinder
    {
        public List<GameTile> [][][][] PossibleTiles = null;
        public int[] matchingLeftCounts = null;
        public int[,] matchingUpCounts = null;
        public int[,,] matchingRightCounts = null;

        public GameTile[] AllTileRotations = null;
        public int TilesCount;
        public int ColorsCount;
        public int FreeTilesLeft = 0;

#if USE_COLOR_COUNTING
        public int [] remainingTilesOfColor = null;
        public int[] unmatchedTilesOfColor = null;
#endif

        public TileFinder(GameTile[] tiles, int colors)
        {
            TilesCount = tiles.Length;            
            FreeTilesLeft = TilesCount;
            ColorsCount = colors;

#if USE_COLOR_COUNTING
            remainingTilesOfColor = new int[ColorsCount];
            unmatchedTilesOfColor = new int[ColorsCount];

            for (int i = 0; i < ColorsCount; ++i)
            {
                remainingTilesOfColor[i] = 0;
                unmatchedTilesOfColor[i] = 0;
            }
            foreach (GameTile tile in tiles)
            {
                ++remainingTilesOfColor[tile.LeftColor];
                ++remainingTilesOfColor[tile.TopColor];
                ++remainingTilesOfColor[tile.RightColor];
                ++remainingTilesOfColor[tile.BottomColor];
            }            
#endif

            matchingLeftCounts = new int[ColorsCount];
            matchingUpCounts = new int[ColorsCount, ColorsCount];
            matchingRightCounts = new int[ColorsCount, ColorsCount, ColorsCount];

            PossibleTiles = new List<GameTile>[ColorsCount][][][];
            for (int iL = 0; iL < ColorsCount; ++iL)
            {
                PossibleTiles[iL] = new List<GameTile>[ColorsCount][][];
                matchingLeftCounts[iL] = 0;
                for (int iU = 0; iU < ColorsCount; ++iU)
                {
                    PossibleTiles[iL][iU] = new List<GameTile>[ColorsCount][];
                    matchingUpCounts[iL,iU] = 0;
                    for (int iR = 0; iR < ColorsCount; ++iR)
                    {
                        PossibleTiles[iL][iU][iR] = new List<GameTile>[ColorsCount];
                        matchingRightCounts[iL,iU,iR] = 0;
                        for (int iD = 0; iD < ColorsCount; ++iD)
                        {
                            PossibleTiles[iL][iU][iR][iD] = new List<GameTile>();
                        }
                    }
                }
            }

            AllTileRotations = new GameTile[TilesCount * 4];

            for (int rot = 0; rot < 4; ++rot)
            {
                for (int i = 0; i < TilesCount; ++i)
                {
                    AllTileRotations[i + rot * tiles.Length] = tiles[i].Clone();
                    AllTileRotations[i + rot * tiles.Length].Rotation = rot;
                }
            }
            for (int i = 0; i < TilesCount; ++i)
            {
                AddFreeTile(tiles[i]);
            }
        }

        public void UpdateTileUniqueness(List<GameTile> tiles)
        {
            foreach (GameTile tile in tiles)
            {
                /*tile.Uniqueness = Math.Min(Math.Min(remainingTilesOfColor[tile.LeftColor], remainingTilesOfColor[tile.TopColor]),
                        Math.Min(remainingTilesOfColor[tile.RightColor], remainingTilesOfColor[tile.BottomColor]));*/

                tile.Uniqueness = remainingTilesOfColor[tile.LeftColor] *  remainingTilesOfColor[tile.LeftColor] * tile.ColorUniquenessMultiplierLeft + 
                                  remainingTilesOfColor[tile.TopColor] *  remainingTilesOfColor[tile.TopColor] * tile.ColorUniquenessMultiplierTop +
                                  remainingTilesOfColor[tile.RightColor] *  remainingTilesOfColor[tile.RightColor] * tile.ColorUniquenessMultiplierRight +
                                  remainingTilesOfColor[tile.BottomColor] *  remainingTilesOfColor[tile.BottomColor] * tile.ColorUniquenessMultiplierBottom;
            }
        }

        public void AddFreeTile(GameTile tile)
        {
            AddRotationOfFreeTile(AllTileRotations[tile.Index + 0 * TilesCount]);
            AddRotationOfFreeTile(AllTileRotations[tile.Index + 1 * TilesCount]);
            AddRotationOfFreeTile(AllTileRotations[tile.Index + 2 * TilesCount]);
            AddRotationOfFreeTile(AllTileRotations[tile.Index + 3 * TilesCount]);
            ++FreeTilesLeft;
        }

        public void RemoveFreeTile(GameTile tile)
        {
            RemoveRotationOfFreeTile(AllTileRotations[tile.Index + 0 * TilesCount]);
            RemoveRotationOfFreeTile(AllTileRotations[tile.Index + 1 * TilesCount]);
            RemoveRotationOfFreeTile(AllTileRotations[tile.Index + 2 * TilesCount]);
            RemoveRotationOfFreeTile(AllTileRotations[tile.Index + 3 * TilesCount]);
            --FreeTilesLeft;
        }

        public void TileIsPlacedOnBoard(GameTile tile, int patternLeft, int patternUp, int patternRight, int patternDown)
        {
#if USE_COLOR_COUNTING
            //left
            --remainingTilesOfColor[tile.LeftColor];
            if (patternLeft == -1)
            {
                ++unmatchedTilesOfColor[tile.LeftColor];
            }
            else
            {
                --unmatchedTilesOfColor[tile.LeftColor];
            }
            // top
            --remainingTilesOfColor[tile.TopColor];
            if (patternUp == -1)
            {
                ++unmatchedTilesOfColor[tile.TopColor];
            }
            else
            {
                --unmatchedTilesOfColor[tile.TopColor];
            }
            // right
            --remainingTilesOfColor[tile.RightColor];
            if (patternRight == -1)
            {
                ++unmatchedTilesOfColor[tile.RightColor];
            }
            else
            {
                --unmatchedTilesOfColor[tile.RightColor];
            }
            //down
            --remainingTilesOfColor[tile.BottomColor];
            if (patternDown == -1)
            {
                ++unmatchedTilesOfColor[tile.BottomColor];
            }
            else
            {
                --unmatchedTilesOfColor[tile.BottomColor];
            }
#endif
        }

        public void TileIsRemovedFromBoard(GameTile tile, int patternLeft, int patternUp, int patternRight, int patternDown)
        {
#if USE_COLOR_COUNTING
            //left
            ++remainingTilesOfColor[tile.LeftColor];
            if (patternLeft == -1)
            {
                --unmatchedTilesOfColor[tile.LeftColor];
            }
            else
            {
                ++unmatchedTilesOfColor[tile.LeftColor];
            }
            // top
            ++remainingTilesOfColor[tile.TopColor];
            if (patternUp == -1)
            {
                --unmatchedTilesOfColor[tile.TopColor];
            }
            else
            {
                ++unmatchedTilesOfColor[tile.TopColor];
            }
            // right
            ++remainingTilesOfColor[tile.RightColor];
            if (patternRight == -1)
            {
                --unmatchedTilesOfColor[tile.RightColor];
            }
            else
            {
                ++unmatchedTilesOfColor[tile.RightColor];
            }
            //down
            ++remainingTilesOfColor[tile.BottomColor];
            if (patternDown == -1)
            {
                --unmatchedTilesOfColor[tile.BottomColor];
            }
            else
            {
                ++unmatchedTilesOfColor[tile.BottomColor];
            }
#endif
        }

        public List<GameTile> GetMatchingTiles(int leftPattern, int upPattern, int rightPattern, int downPattern)
        {
            List<GameTile> matchingTiles = new List<GameTile>(4 * FreeTilesLeft);
            GetTilesForLeftPattern(PossibleTiles, leftPattern, upPattern, rightPattern, downPattern, matchingTiles);
            return matchingTiles;
        }

        private void GetTilesForLeftPattern(List<GameTile>[][][][] root, int leftColor, int upColor, int rightColor, int downColor, List<GameTile> matches)
        {
            if (leftColor == -1)
            {
                for (int i = 1; i < ColorsCount; ++i )
                {
                    if (matchingLeftCounts[i] == 0) continue;
                    GetTilesForUpPattern(root[i], i, upColor, rightColor, downColor, matches);
                }

                return;
            }
            GetTilesForUpPattern(root[leftColor], leftColor, upColor, rightColor, downColor, matches);            
        }

        private void GetTilesForUpPattern(List<GameTile>[][][] root, int leftColor, int upColor, int rightColor, int downColor, List<GameTile> matches)
        {
            if (upColor == -1)
            {
                for (int i = 1; i < ColorsCount; ++i)
                {
                    if (matchingUpCounts[leftColor, i] == 0) continue;
                    GetTilesForRightPattern(root[i], leftColor, i, rightColor, downColor, matches);            
                }
                return;
            }
            GetTilesForRightPattern(root[upColor], leftColor, upColor, rightColor, downColor, matches);            
        }

        private void GetTilesForRightPattern(List<GameTile>[][] root, int leftColor, int upColor, int rightColor, int downColor, List<GameTile> matches)
        {
            if (rightColor == -1)
            {
                for (int i = 1; i < ColorsCount; ++i)
                {
                    if (matchingRightCounts[leftColor, upColor, i] == 0) continue;
                    GetTilesForDownPattern(root[i], leftColor, upColor, i, downColor, matches);            
                }
                return;
            }
            GetTilesForDownPattern(root[rightColor], leftColor, upColor, rightColor, downColor, matches);            
        }

        public bool PlaceTileIfPossible(GameTile tile, int leftColor, int upColor, int rightColor, int downColor)
        {            
#if USE_COLOR_COUNTING            
            TileIsPlacedOnBoard(tile, leftColor, upColor, rightColor, downColor);
            if ((unmatchedTilesOfColor[tile.LeftColor] <= remainingTilesOfColor[tile.LeftColor]) &&
                (unmatchedTilesOfColor[tile.TopColor] <= remainingTilesOfColor[tile.TopColor]) &&
                (unmatchedTilesOfColor[tile.RightColor] <= remainingTilesOfColor[tile.RightColor]) &&
                (unmatchedTilesOfColor[tile.BottomColor] <= remainingTilesOfColor[tile.BottomColor]))
            {
                return true;
            }
            TileIsRemovedFromBoard(tile, leftColor, upColor, rightColor, downColor);
            return false;
#else
            TileIsPlacedOnBoard(tile, leftColor, upColor, rightColor, downColor);
            return true;
#endif
        }

        private void GetTilesForDownPattern(List<GameTile>[] root, int leftColor, int upColor, int rightColor, int downColor, List<GameTile> matches)
        {
            if (downColor == -1)
            {
                for (int i = 1; i < ColorsCount; ++i)
                {
                    if (root[i].Count == 0) continue;
                    matches.AddRange(root[i]);
                }
                return;
            }
            matches.AddRange(root[downColor]);
        }

        private void AddRotationOfFreeTile(GameTile tile)
        {
            PossibleTiles[tile.LeftColor][tile.TopColor][tile.RightColor][tile.BottomColor].Add(tile);            
            ++matchingLeftCounts[tile.LeftColor];
            ++matchingUpCounts[tile.LeftColor, tile.TopColor];
            ++matchingRightCounts[tile.LeftColor, tile.TopColor, tile.RightColor];
        }

        private void RemoveRotationOfFreeTile(GameTile tile)
        {
            PossibleTiles[tile.LeftColor][tile.TopColor][tile.RightColor][tile.BottomColor].Remove(tile);
            --matchingLeftCounts[tile.LeftColor];
            --matchingUpCounts[tile.LeftColor, tile.TopColor];
            --matchingRightCounts[tile.LeftColor, tile.TopColor, tile.RightColor];
        }
    }
}
